#ifndef __COMMON_H
#define __COMMON_H

#ifndef __KERNEL__                       // only for user space applications
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <netinet/in.h>  
#include <linux/netdevice.h>

#define ERR(...)  fflush( stdout ), fprintf( stderr, __VA_ARGS__ ), fflush( stderr ), exit( EXIT_FAILURE )

#else                                    // for kernel space module
#include <linux/module.h>
#include <linux/version.h>
#include <linux/init.h>
#include <linux/kallsyms.h>
#include <linux/uaccess.h>
#include <linux/unistd.h>
#include <linux/miscdevice.h>
#include <linux/inet.h>
#include <linux/netdevice.h>

#define PREFIX "! "
#define DEB(...) if( debug ) printk( KERN_INFO PREFIX __VA_ARGS__ )
#define LOG(...) printk( KERN_INFO PREFIX __VA_ARGS__ )
#define ERR(...) printk( KERN_ERR PREFIX __VA_ARGS__ )

inline char* in4_ntoa( uint32_t ip ) {   // mapping IP to a string
   static char saddr[ MAX_ADDR_LEN ];
   sprintf( saddr, "%d.%d.%d.%d",
            ( ip >> 24 ) & 0xFF, ( ip >> 16 ) & 0xFF, 
            ( ip >> 8 ) & 0xFF, ( ip ) & 0xFF
          );
   return saddr;
}

static bool debug = false;               // logical parameter: debug output level, 0/1
module_param( debug, bool, 0 );

#endif // __KERNEL__

#define DEVNAME "policy"                 // device file name in /dev
#define LEN_MSG 256                      // text messages max length

typedef enum syscall_type { ACCEPT_BEFORE, CONNECT, ACCEPT_AFTER } syscall_type_t;

typedef struct {
   uint32_t sequence;
   syscall_type_t type;
   uint32_t       ipn;   
   uint16_t       portn; 
   char text[ LEN_MSG ];
} policy_mess_t;

typedef enum permission_type { DENY=0, ALLOW } permission_type_t;

typedef struct {
   uint32_t          sequence;
   permission_type_t type;
} policy_perm_t;

#endif // __COMMON_H
