#include "../common.h"
#include <time.h>
#include <sys/timeb.h>
#include <arpa/inet.h>

/*   struct timeb {
               time_t         time;
               unsigned short millitm;
               short          timezone;
               short          dstflag;
           }; */
/*         struct tm {
               int tm_sec;         // seconds 
               int tm_min;         // minutes 
               int tm_hour;        // hours 
               int tm_mday;        // day of the month 
               int tm_mon;         // month 
               int tm_year;        // year 
               int tm_wday;        // day of the week 
               int tm_yday;        // day in the year 
               int tm_isdst;       // daylight saving time 
           }; */
char* time_mark( void ) {
   static char ret[ 10 ];
   struct timeb tb;
   ftime( &tb );
  struct tm *lt = localtime( &tb.time );
  sprintf( ret, "%02d:%02d.%03d", lt->tm_min, lt->tm_sec, tb.millitm % 1000 );
  return ret;
}

char* policy2str( policy_mess_t pol ) {
   static char msg[ LEN_MSG ];
   switch( pol.type ) {
      case ACCEPT_BEFORE: strcpy( msg, "accept wait: " ); break;
      case CONNECT:       strcpy( msg, "connect to:  " ); break;
      case ACCEPT_AFTER:  strcpy( msg, "accept from: " ); break;
      default :           strcpy( msg, "undef.type:  " );
   } 
   sprintf( msg + strlen( msg ), "sequence %04d | ", pol.sequence );   
   struct in_addr ip = { pol.ipn };
   sprintf( msg + strlen( msg ), "IP=%15s | TCP PORT=%u\t| %s",
            inet_ntoa( ip ), ntohs( pol.portn ),
            pol.text  );   
   return msg;
}

int main( int argc, char *argv[] ) {
   char dev[ 80 ] = "/dev/", c, sopt[] = "v";
   int df, debug_level = 0;
   while( -1 != ( c = getopt( argc, argv, sopt ) ) )
      switch( c ) {
         case 'v':
            debug_level++;
            break;
         default :
            fprintf( stderr, "use: %s [-v[v...]]\n", argv[ 0 ] );
            exit( EXIT_FAILURE );
      }
   strcat( dev, DEVNAME );
   if( ( df = open( dev, O_RDWR ) ) < 0 )
      fprintf( stderr, "open device %s error: %m\n", dev ), exit( EXIT_FAILURE );
   int res;
   do {
      policy_mess_t policy;
      policy_perm_t permission;
      res = read( df, &policy, sizeof( policy_mess_t ) );
      if( res < 0 ) {
         fprintf( stderr, "read device error: %m\n" );
         break;
      }
      if( sizeof( policy_mess_t ) == res ) {
         if( debug_level ) printf( "[%s] %s\n", time_mark(), policy2str( policy ) );
         else printf( "%s\n", policy2str( policy ) ); 
      }
      else 
	 printf( "read %d bytes\n", res );
      permission.sequence = policy.sequence;
      permission.type = sizeof( policy_mess_t ) == res ? ALLOW : DENY;
      res = write( df, &permission, sizeof( policy_perm_t ) );
      if( res != sizeof( policy_perm_t ) ) {
         fprintf( stderr, "write device error: %m\n" );
         break;
      }
      printf( "write %d bytes\n", res );
   } while ( res > 0 );
   close( df );
   return EXIT_SUCCESS;
};
