#include "../common.h"
#include "../find.c"

/* <linux/syscalls.h>:
asmlinkage long sys_connect(int, struct sockaddr __user *, int);
asmlinkage long sys_accept(int, struct sockaddr __user *, int __user *);
asmlinkage long sys_socketcall(int call, unsigned long __user *args);
*/

/* man pages:
#include <sys/socket.h>
int accept(int sockfd, struct sockaddr *addr, socklen_t *addrlen);
int connect(int sockfd, const struct sockaddr *addr, socklen_t addrlen);
int socketcall(int call, unsigned long *args);
*/

asmlinkage long (*old_sys_write) ( unsigned int fd, const char __user *buf, size_t count );

static char buf[160 ];                       // messages for output

static int write( void ) {                   // put message in tty
   int n;
   mm_segment_t fs = get_fs();
   set_fs( get_ds() );
   n = old_sys_write( 1, buf, strlen( buf ) );
   set_fs( fs );
   return n;
}

#define printf( format, ... )  sprintf( buf, format, __VA_ARGS__ ); write();

static void **taddr;                         // table sys_call_table address
const int last = __NR_process_vm_writev;     // last syscall in i386;

static void show_in_table( const char *symbol ) {
   int n;
   void *waddr = find_sym( symbol );
   for( n = 0; n <= last; n++ )
      if( taddr[ n ] == waddr ) break;
   if( n <= last )
      sprintf( buf, "symbol: %s address=%p found in position %d\n", symbol, waddr, n );
   else
      sprintf( buf, "symbol: %s address=%p not found\n", symbol, waddr );
   write();
}

static int __init init( void ) {
   void *waddr;
   printk ( "--------------------------------------------------------------------\n" );
   if( NULL == ( taddr = find_sym( "sys_call_table" ) ) ) {
      ERR( "sys_call_table not found\n" ); return -EINVAL;
   }
   old_sys_write = (void*)taddr[ __NR_write ];
   if( NULL == ( waddr = find_sym( "sys_write" ) ) ) {
      ERR( "sys_write not found\n" ); return -EINVAL;
   }
   if( old_sys_write != waddr ) {
      ERR( "Oooops! : addresses not equal\n" ); return -EINVAL;
   }
   printf( "address of sys_call_table  = %p\n", taddr );
   printf( "size of sys_call_table  = %d\n", last + 1 );
   printf( "__NR_socketcall = %d\n", __NR_socketcall );
   show_in_table( "sys_write" );
   show_in_table( "sys_accept" );
   show_in_table( "sys_connect" );
   show_in_table( "sys_socketcall" );
   return -EPERM;    // do not need installation
}

module_init( init );

MODULE_LICENSE( "GPL" );
MODULE_AUTHOR( "Oleg Tsiliuric <olej.tsil@gmail.com>" );

